function Kd_sim_estimate(configfile)

[datapath, result_prefix, nTime, width, height, timefile, domefile, timestamp, description] = load_config_file(configfile);
timeV = load_sample_timefile(timefile,nTime);
timeV = timeV/max(timeV); % normalize time to be in [0,1]

%-----------------------------------------
%%% Step 1: 
% loading files
I_buf_R = zeros(height, width, nTime);
I_buf_G = zeros(height, width, nTime);
I_buf_B = zeros(height, width, nTime);

fprintf('loading files ');
for time = 0:nTime-1
    filename = sprintf('%s\\TSparamNew\\%s-%02d-xx-TSparam-Kd.exr', datapath, result_prefix, time);   
    img      = exrread(filename);        
    I_buf_R(:,:,time+1) = img(:,:,1);
    I_buf_G(:,:,time+1) = img(:,:,2);
    I_buf_B(:,:,time+1) = img(:,:,3);
        
    fprintf('.');
end
fprintf('done.\n');
clear img;

%-----------------------------------------
%%% Step 2:
% estimate the structural average curve
timeV = timeV';
border_w = 20;  
w = round((width-2*border_w)/20); % make sure about 400 sample points to estimate the curve \phi(t')
  
m = 1;
for i=border_w : w : height-border_w
    for j=border_w : w : width-border_w
        a = I_buf_R(i,j,:); a = a(:); a = a';
        x(m,:) = a;
        m = m + 1;
    end
end
% current algorithm (polyfit, time range >[0,1], do normalization)
% [strav_r, bopt_r, strav_t_r, theta_r, pphi_r] = sim_estimate(timeV,x,1,6,5,0,1);

% previous work algorithm (polyfit, time range =[0,1], no normalization)
%[strav_r, bopt_r, strav_t_r, theta_r, pphi_r] = sim_estimate(timeV,x,1,6,5,2,0);
[strav_r, bopt_r, strav_t_r, theta_r, pphi_r] = sim_estimate(timeV,x,1,6,5,2,1);

% algorithm in the original paper (no polyfit, time range<[0,1], do norm)
%[strav_r, bopt_r, strav_t_r, theta_r, pphi_r] = sim_estimate(timeV,x,0,6,5,1,1);

m = 1;
for i=border_w : w : height-border_w
    for j=border_w : w : width-border_w
        a = I_buf_G(i,j,:); a = a(:); a = a';
        x(m,:) = a;
        m = m + 1;
    end
end
% [strav_g, bopt_g, strav_t_g, theta_g, pphi_g] = sim_estimate(timeV,x,1,6,5,0,1);
% [strav_g, bopt_g, strav_t_g, theta_g, pphi_g] = sim_estimate(timeV,x,1,6,5,2,0);
[strav_g, bopt_g, strav_t_g, theta_g, pphi_g] = sim_estimate(timeV,x,1,6,5,2,1);

% [strav_g, bopt_g, strav_t_g, theta_g, pphi_g] = sim_estimate(timeV,x,0,6,5,1,1);

m = 1;
for i=border_w : w : height-border_w
    for j=border_w : w : width-border_w
        a = I_buf_B(i,j,:); a = a(:); a = a';
        x(m,:) = a;
        m = m + 1;
    end
end
% [strav_b, bopt_b, strav_t_b, theta_b, pphi_b] = sim_estimate(timeV,x,1,6,5,0,1);
% [strav_b, bopt_b, strav_t_b, theta_b, pphi_b] = sim_estimate(timeV,x,1,6,5,2,0);
[strav_b, bopt_b, strav_t_b, theta_b, pphi_b] = sim_estimate(timeV,x,1,6,5,2,1);

% [strav_b, bopt_b, strav_t_b, theta_b, pphi_b] = sim_estimate(timeV,x,0,6,5,1,1);

clear x;

filename = sprintf('%s\\SIM\\%s-strav.mat', datapath, result_prefix);
save(filename, 'strav_r', 'strav_g', 'strav_b', 'bopt_r', 'bopt_g', 'bopt_b', ... 
               'strav_t_r', 'strav_t_g', 'strav_t_b', 'pphi_r', 'pphi_g', 'pphi_b', 'timeV');

% save the estimate curve \phi(t)
h = figure(1);
hold on;
plot(strav_t_r, strav_r, '-r', 'LineWidth', 2);
plot(strav_t_g, strav_g, '-g', 'LineWidth', 2);
plot(strav_t_b, strav_b, '-b', 'LineWidth', 2);
title('structural average curve');
xlabel('time');
legend('Red', 'Green', 'Blue', 0);
filename = sprintf('%s\\SIM\\%s_strav.jpg', datapath, result_prefix);
print(h, '-djpeg', filename);
close(h);

%-------------------------------------------------
%%% Step 3-1
% use the previous estimation and interpolate to
% get a good initial value for each point

[XI,YI]    = meshgrid(border_w : w : width-border_w, border_w : w : height-border_w);
ZI_theta_R = zeros(size(XI,1), size(XI,2), 4);
ZI_theta_G = zeros(size(XI,1), size(XI,2), 4);
ZI_theta_B = zeros(size(XI,1), size(XI,2), 4);
m = 1;
for i=1:size(XI,1)
    for j=1:size(XI,2)
		ZI_theta_R(i,j,:) = theta_r(m,:);
        ZI_theta_G(i,j,:) = theta_g(m,:);
        ZI_theta_B(i,j,:) = theta_b(m,:);
        m = m+1;
    end
end
clear theta_r theta_g theta_b;

theta_R = zeros(height, width, 4);
theta_G = zeros(height, width, 4);
theta_B = zeros(height, width, 4);
extraval = [1,1,0,0]; % initial values for A,B,C,D is 1,1,0,0 for the border region
[X,Y] = meshgrid(1:width, 1:height);
for i = 1:4
    theta_R(:,:,i) = interp2(XI,YI,ZI_theta_R(:,:,i),X,Y,'linear',extraval(i));
    theta_G(:,:,i) = interp2(XI,YI,ZI_theta_G(:,:,i),X,Y,'linear',extraval(i));
    theta_B(:,:,i) = interp2(XI,YI,ZI_theta_B(:,:,i),X,Y,'linear',extraval(i));
end
clear ZI_theta_R ZI_theta_G ZI_theta_B XI YI X Y;

%-------------------------------------------------
%%% Step 3-2
% estimate the theta parameters (A,B,C,D) for each point

options = optimset('Display', 'Off');
errV = zeros(height, width, 3);

ttt = clock;
for i = border_w : height-border_w
    for j = border_w : width-border_w
        
        fprintf('i %d j %d\n', i, j);
        
        x = I_buf_R(i,j,:); x = x(:); x = x';   
        p0 = theta_R(i,j,:); p0 = p0(:); p0 = p0';
        [p, err, res, flag] = lsqnonlin(@myfun, p0, [0,0,-inf,-inf], [inf,inf,inf,inf], options, timeV, x, pphi_r);
        theta_R(i,j,:) = p;
        fprintf('r\t%f\t%f\t%f\t%f\terr %f\tflag %d\n',p(1),p(2),p(3),p(4),err,flag);
        errV(i,j,1) = err;
        
        x = I_buf_G(i,j,:); x = x(:); x = x';        
        p0 = theta_G(i,j,:); p0 = p0(:); p0 = p0';
        [p, err, res, flag] = lsqnonlin(@myfun, p0, [0,0,-inf,-inf], [inf,inf,inf,inf], options, timeV, x, pphi_g);
        theta_G(i,j,:) = p;
        fprintf('g\t%f\t%f\t%f\t%f\terr %f\tflag %d\n',p(1),p(2),p(3),p(4),err,flag);        
        errV(i,j,2) = err;

        x = I_buf_B(i,j,:); x = x(:); x = x';        
        p0 = theta_B(i,j,:); p0 = p0(:); p0 = p0';
        [p, err, res, flag] = lsqnonlin(@myfun, p0, [0,0,-inf,-inf], [inf,inf,inf,inf], options, timeV, x, pphi_b);
        theta_B(i,j,:) = p;
        fprintf('b\t%f\t%f\t%f\t%f\terr %f\tflag %d\n',p(1),p(2),p(3),p(4),err,flag);
        errV(i,j,3) = err;
        
    end
end
filename = sprintf('%s\\SIM\\%s-theta.mat', datapath, result_prefix);
save(filename, 'theta_R', 'theta_G', 'theta_B', 'errV');
fprintf('time %d\n',etime(clock,ttt));
clear all;

%---------------------------------------------
function f = myfun(p, t, x, pphi)

z = polyval(pphi, (t-p(3))/p(2));
f = x - p(1)*z - p(4);
f = f';
